// Define the URL you want to send the POST request to:
// const url = 'https://api.osv.dev/v1/query';
const url = 'https://osv-proxy.requiresecurity.com';

// Define the request headers (typically, you set the 'Content-Type' header for
// JSON data):
const headers = {
  'Content-Type': 'application/json'
};

const requestQueue = [];
const retryMap = new Map();

const REQ_DELAY = 130;

function SendRequest(libId, reqData) {
  return new Promise((resolve, reject) => {
    const body = JSON.stringify(reqData);

    // Create the fetch() request
    fetch(url, {
      method: 'POST',
      headers: headers,
      body: body
    })
    .then(response => {
      if (!response.ok) {
        throw new Error(`Network response was not ok (status = ${response.status})`);
      }
      return response.json();
    })
    .then(responseData => {
      if (Object.keys(responseData).length > 0) {
        self.postMessage({ libId, responseData });
      }
      resolve(true);  // Received response from OSV server, so resolve successfully.
    })
    .catch(error => {
      console.warn(`Error while posting CVE query (${body}): ${error}`);
      resolve(false); // Response was not OK, so resolve unsuccessfully.
    });
  });
}

// Listen for events coming through from the parent.
self.onmessage = function (event) {
  // Post a message back to the vulnerability store indicating that there are
  // more requests to process.
  self.postMessage({hasRequest: true});

  // Push a new request onto the request queue.
  requestQueue.unshift(event.data);
};

async function HandleRequest() {
  if (requestQueue.length > 0) {
    // Process a single request.
    const data = requestQueue.pop();
    SendRequest(data.libId, {
      package: {
        name: data.libName,
        ecosystem: "npm"
      },
      version: data.libVersion
    }).then(success => {
      const key = `${data.libId},${data.libName}@${data.libVersion}`;
      if (success) {
        // If there was an entry in the retry map, then remove it since we have
        // successfully processed the request for this library/version.
        retryMap.delete(key);
      } else {
        // Un oh, the request failed for some reason.  Let's try again (up to
        // three times if necessary).
        let count = 1;
        if (retryMap.has(key)) {
          count = retryMap.get(key) + 1;
        }
        if (count > 3) {
          console.error(`Unable to process request for '${key}' after 3 retries`);
        } else {
          // Update the retry map with the next count and push the data back
          // into the queue to be tried again at a later point.
          retryMap.set(key, count);
          requestQueue.unshift(data);
        }
      }
    });
  } else {
    // No more requests... Update the vulnerability store by posting a
    // message back to it.
    self.postMessage({ hasRequest: false });
  }

  // Schedule the next call to HandleRequest after an appropriate delay.
  setTimeout(HandleRequest, REQ_DELAY);
}

HandleRequest();